################################################################
# Data Analysis Code - Lightweight Relief Shearing for Enhanced Terrain Perception on Interactive Maps (CHI '15)
# Performs data processing and generates plots for Experiment 2
# 2014 Wesley Willett
################################################################

# The Directory containing all R scripts and data files.
# SET THIS! --- Should point to the current absolute path to the directory containing the R scripts and data files.
sourceDir <- "YOUR_SOURCE_DIRECTORY"

setwd(sourceDir)
source("bootstrap_macros.R")
source("plotting.R")

#############################################################################

# Load raw data file
files <- "log-experiment2.csv"
datasetName <- "Experiment 2"

score_formatter <- function(x) {
    label <- sprintf('%.2f', x)
    return (label)
}

#############################################################################

# Columns of the cleaned up table
c_participant <- NULL
c_tech <- NULL
c_map <- NULL
c_correct <- NULL
c_distance <- NULL
c_elevation <- NULL
c_trialnumber <- NULL
c_cumerrors <- NULL
c_time <- NULL

for (file in files) {
	rawdata <- read.table(file,header=TRUE,sep=",")

	# FIXME: Loops are ugly in R and we should vectorize instead
	for(i in 1:nrow(rawdata)) {
	    row <- rawdata[i,]
	    # Compute cell values
	    participant <- as.character(row$Participant.Id)
		
	    # Skip rows that don't contain a red/blue response (e.g. pauses/skips)
	    if (row$Response != "red" && row$Response != "blue"){ next }
		
		# Participant filter - keep only the participants for Experiment 2
		if (as.integer(participant) < 50 || as.integer(participant) > 70) { next }

		# Time filter - remove erroneous < 1s responses 
		if (as.integer(row$Task.Time) < 0.6) { next }

	    # Tech
	    tech <- as.character(row$Condition)
	    if (!(tech %in% c("Static","Angled-Perspective","Top-Perspective","Orientation","Implicit"))) { next }
	    
	    # Map
	    filename <- row$Grid.Source
	    filenameregex <- ".*/([^/]*)\\.grd$"
	    map <- gsub(filenameregex, "\\1", filename, perl = TRUE)
	    if (map == "BernAlps-Study"){ next }

		# Elevation and Distance
		elevation <- row$Abs.Elevation.Diff
		distance <- row$Abs.DistanceXY	
		
		# Trial Number and Cumulative Errors
		trialnumber <- row$Trial.Number
		cumerrors <- row$Cumulative.Errors
	   
	    # Correct
	    correct <- ifelse((row$RedZ > row$BlueZ && row$Response == "red") || (row$BlueZ > row$RedZ && row$Response == "blue"), 1, 0)
	    time <- row$Task.Time
	    # Update columns
	    c_participant <- append(c_participant, participant)
	    c_tech <- append(c_tech, tech)
	    c_map <- append(c_map, map)
	    c_correct <- append(c_correct, correct)
	    c_elevation <- append(c_elevation, elevation)
	    c_distance <- append(c_distance, distance)
	    c_trialnumber <- append(c_trialnumber, trialnumber)
	    c_cumerrors <- append(c_cumerrors, cumerrors)
	    c_time <- append(c_time, time)
	}
}

# Build cleaned-up data frame
data <- data.frame(
	factor(c_participant),
	factor(c_map),
	factor(c_tech),
	factor(c_correct),
	c_elevation,
	c_distance,
	c_trialnumber,
	c_cumerrors,
	c_time)
colnames(data) <- c("participant", "map", "tech", "correct", "elevation", "distance", "trialnumber", "cumerrors", "time")


#############################################################################
# COMPARING TECHNIQUES
# Compare each participant's accuracy rate across each of the techniques

c_tech <- NULL
c_map <- NULL
c_mean <- NULL
c_cimin <- NULL
c_cimax <- NULL

# Compare techniques across all maps
for (tech in levels(data$tech)) {
	scores <- NULL
	for (participant in levels(data$participant)) {
		success <- data$correct[data$participant == participant & data$tech == tech]
		#score <- length(success[success ==1]) / length(success)  # Computes crude %correct
		score <- (percentCI(success, 1)[1] - 50) / 50 # Computes a CI, takes the mean, then normalizes
		scores <- append(scores, score)
	}
	mean <- bootstrapMeanCI(scores) # changed from meanCI into bootstrap now that we have more subjects
	c_map <- append(c_map, "All maps")
	c_tech <- append(c_tech, tech)
	c_mean <- append(c_mean, mean[1])
	c_cimin <- append(c_cimin, mean[2])
	c_cimax <- append(c_cimax, mean[3])
}

allScoresAnalysis <- data.frame(
	factor(c_map),
	factor(c_tech, levels = c("Static","Angled-Perspective","Top-Perspective","Orientation","Implicit"),
				   labels = c("Standard Panning","Inclined Pers.","Top-Down Pers.","Device Tilt","Integrated Shearing")), # reorder factors
	c_mean, c_cimin, c_cimax)
colnames(allScoresAnalysis) <- c("map", "tech", "pointEstimate", "ci.min", "ci.max")
print(allScoresAnalysis)

#plot
hplot(allScoresAnalysis, "Accuracy", datasetName, "tech", 0, 1, score_formatter, height=1.4)


#############################################################################
# COMPARE TECHNIQUE PAIRS

c_techs <- NULL
c_mean <- NULL
c_cimin <- NULL
c_cimax <- NULL

techsInOrder = c("Static","Angled-Perspective","Top-Perspective","Orientation","Implicit")

# By  technique
for (t1 in seq(length(techsInOrder))) {
	tech1 <- techsInOrder[t1]
	for (t2 in seq(t1, length(techsInOrder))) {
		tech2 <- techsInOrder[t2]
		if (tech1 == tech2 ){ next }
		scores <- NULL
		for (part in levels(data$participant)) {
			results1 <- data$correct[data$participant == part & data$tech == tech1]
			results2 <- data$correct[data$participant == part & data$tech == tech2]
			correct1 <- (percentCI(results1, 1)[1] - 50) / 50
			correct2 <- (percentCI(results2, 1)[1] - 50) / 50
			score <- correct2 - correct1
			scores <- append(scores, score)
		}
		meanci <- bootstrapMeanCI(scores) # changed from meanCI into bootstrap now that we have more subjects
		c_techs <- append(c_techs, paste(tech2,"-",tech1))
		c_mean <- append(c_mean, meanci[1])
		c_cimin <- append(c_cimin, meanci[2])
		c_cimax <- append(c_cimax, meanci[3])
	}
}

techPairsAnalysis <- data.frame(
	factor(c_techs, levels = c("Orientation - Static","Angled-Perspective - Static","Top-Perspective - Static","Top-Perspective - Angled-Perspective","Orientation - Angled-Perspective","Orientation - Top-Perspective","Implicit - Orientation","Implicit - Top-Perspective","Implicit - Angled-Perspective","Implicit - Static"),
	labels = c("Device Tilt - Standard","Inc. Pers. - Standard","Top Pers. - Static","Top Pers. - Inc. Pers.","Device Tilt - Inc. Pers.","Device Tilt - Top Pers.","Integrated - Device Tilt","Integrated - Top Pers.","Integrated - Inc. Pers.","Integrated - Standard")), 
	c_mean, c_cimin, c_cimax)
colnames(techPairsAnalysis) <- c("techs", "pointEstimate", "ci.min", "ci.max")

#filter
techPairsToShow <- c("Device Tilt - Standard","Integrated - Standard","Integrated - Inc. Pers.","Integrated - Top Pers.","Integrated - Device Tilt")
techPairsAnalysis <- techPairsAnalysis[techPairsAnalysis $techs %in% techPairsToShow,]

#sort
#techPairsAnalysis$techs <- reorder(techPairsAnalysis$techs, techPairsAnalysis$pointEstimate)

# plot
#hplot(techPairsAnalysis, "Improvement in Accuracy", datasetName, "techs", -0.1, 0.3, score_formatter, height=2)
hplot(techPairsAnalysis, "Improvement in Accuracy", datasetName, "techs", 0, 0.32, score_formatter, height=1.4, width=4.3)

#############################################################################
# LEARNING EFFECTS

# ---- Compute accuracy for every trial block ----

c_trial <- NULL
c_trialcnt <- NULL
c_mean <- NULL
c_cimin <- NULL
c_cimax <- NULL

num_trials <- 15
tasks_before_first <- 50
tasks_per_trial <-20

for (trial in seq(1,num_trials)) {
	scores <- subset(data, trialnumber > tasks_before_first + (trial - 1) * tasks_per_trial & 
							   trialnumber < tasks_before_first + trial * tasks_per_trial)$correct
	mean <- (percentCI(scores, 1)[1] - 50) / 50 
	c_trial <- append(c_trial, trial)
	c_trialcnt <- append(c_trialcnt, length(c_trial))
	c_mean <- append(c_mean, mean[1])
	c_cimin <- append(c_cimin, mean[2])
	c_cimax <- append(c_cimax, mean[3])
}
learningByTrialsAnalysis <- data.frame(
	factor(c_trial),
	c_trialcnt,
	c_mean, c_cimin, c_cimax)
colnames(learningByTrialsAnalysis) <- c("trial", "trialcnt", "pointEstimate", "ci.min", "ci.max")

# Compute Regression
lm(pointEstimate ~ trialcnt, learningByTrialsAnalysis)

# Summary plots (not really necessary)
#fit <- lm(pointEstimate ~ trialcnt, learningByTrialsAnalysis)
#summary(fit) # show results
#layout(matrix(c(1,2,3,4),2,2)) # optional 4 graphs/page 
#plot(fit)


# ---- Compute accuracy for every trial block for each participant ---- 
c_trial <- NULL
c_trialcnt <- NULL
c_mean <- NULL
c_cimin <- NULL
c_cimax <- NULL
num_trials <- 15
tasks_before_first <- 50
tasks_per_trial <-20

for (trial in seq(1,num_trials)) {
	scores <- NULL
	for (part in levels(data$participant)) {
		trialblock = subset(data, participant == part & trialnumber > tasks_before_first + (trial - 1) * tasks_per_trial &
																trialnumber < tasks_before_first + trial * tasks_per_trial)
		success <- trialblock$correct
		score <- (percentCI(success, 1)[1] - 50) / 50
		scores <- append(scores, score)
	}
	mean <- bootstrapMeanCI(scores)
	c_trial <- append(c_trial, trial)
	c_trialcnt <- append(c_trialcnt, length(c_trial))
	c_mean <- append(c_mean, mean[1])
	c_cimin <- append(c_cimin, mean[2])
	c_cimax <- append(c_cimax, mean[3])
}
learningByParTrialsAnalysis <- data.frame(
	factor(c_trial),
	c_trialcnt,
	c_mean, c_cimin, c_cimax)
colnames(learningByParTrialsAnalysis) <- c("trial", "trialcnt", "pointEstimate", "ci.min", "ci.max")

# Compute Regression
lm(pointEstimate ~ trialcnt, learningByParTrialsAnalysis)

# Summary plots (not really necessary)
#fit <- lm(pointEstimate ~ trialcnt, byTrialsAnalysis)
#summary(fit) # show results
#layout(matrix(c(1,2,3,4),2,2)) # optional 4 graphs/page 
#plot(fit)

#############################################################################
# TIME

c_tech <- NULL
c_mean <- NULL
c_cimin <- NULL
c_cimax <- NULL

for (tech in levels(data$tech)) {
	meanTimes <- NULL
	for (participant in levels(data$participant)) {
		times <- data$time[data$participant == participant & data$tech == tech]
		meanTime <- exactMeanCI(times)
		meanTimes <- append(meanTimes, meanTime)
	}
	mean <- bootstrapMeanCI(meanTimes) # changed from meanCI into bootstrap now that we have more subjects
	c_tech <- append(c_tech, tech)
	c_mean <- append(c_mean, mean[1])
	c_cimin <- append(c_cimin, mean[2])
	c_cimax <- append(c_cimax, mean[3])
}

timeAnalysis <- data.frame(
	factor(c_tech, levels = c("Static","Angled-Perspective","Top-Perspective","Orientation","Implicit"),
				   labels = c("Standard Panning","Inclined Pers.","Top-Down Pers.","Device Tilt","Integrated Shearing")), # reorder factors
	c_mean, c_cimin, c_cimax)
colnames(timeAnalysis) <- c("tech", "pointEstimate", "ci.min", "ci.max")
timeAnalysis$tech <- reorder(timeAnalysis$tech, timeAnalysis$pointEstimate)
print(timeAnalysis)

# plot
second_formatter <- function(x) {
    label <- sprintf('%g s', x)
    return (label)
}
hplot(timeAnalysis, "Time", datasetName, "tech", 0, 8, second_formatter, height=1.4)



#############################################################################
# TIME DIFFERENCE BY TECHNIQUE PAIRS

c_techs <- NULL
c_mean <- NULL
c_cimin <- NULL
c_cimax <- NULL

techsInOrder = c("Static","Top-Perspective","Angled-Perspective","Implicit","Orientation")

# By  technique
for (t1 in seq(length(techsInOrder))) {
	tech1 <- techsInOrder[t1]
	for (t2 in seq(t1, length(techsInOrder))) {
		tech2 <- techsInOrder[t2]
		if (tech1 == tech2 ){ next }
		meanTimes <- NULL
		for (part in levels(data$participant)) {
			times1 <- data$time[data$participant == part & data$tech == tech1]
			times2 <- data$time[data$participant == part & data$tech == tech2]
			mean1 <- exactMeanCI(times1)
			mean2 <- exactMeanCI(times2)
			meanTime <- mean2 - mean1
			meanTimes <- append(meanTimes, meanTime)
		}
		meanci <- bootstrapMeanCI(meanTimes)
		c_techs <- append(c_techs, paste(tech2,"-",tech1))
		c_mean <- append(c_mean, meanci[1])
		c_cimin <- append(c_cimin, meanci[2])
		c_cimax <- append(c_cimax, meanci[3])
	}
}

techPairsTimeAnalysis <- data.frame(
	factor(c_techs, levels = c("Angled-Perspective - Static","Top-Perspective - Static","Orientation - Static","Implicit - Static","Angled-Perspective - Top-Perspective","Orientation - Angled-Perspective","Implicit - Angled-Perspective","Orientation - Top-Perspective","Implicit - Top-Perspective","Orientation - Implicit"),
	labels = c("Angled Pers. - Static","Top Pers. - Static","Orient. - Static","Implicit - Static","Angled Pers. - Top Pers.","Orient. - Angled Pers.","Implicit - Angled Pers.","Orient. - Top-Pers.","Implicit - Top Pers.","Orient. - Implicit")), 
	c_mean, c_cimin, c_cimax)
colnames(techPairsTimeAnalysis) <- c("techs", "pointEstimate", "ci.min", "ci.max")
techPairsTimeAnalysis$techs <- reorder(techPairsTimeAnalysis$techs, techPairsTimeAnalysis$pointEstimate)
print(techPairsTimeAnalysis)

# plot
hplot(techPairsTimeAnalysis, "Improvement in Time", datasetName, "techs", 0, 3, second_formatter, height=2)

#############################################################################
# AGES & TIMING

ages <- c(35,39,38,40,24,26,27,28,29,30,29,24)
mean(ages)
sd(ages)


minutesToComplete <- c(75,45,65,65,45,42,39,51,38,53,42,55,50,75)
mean(minutesToComplete)
sd(minutesToComplete)

